import { Observable, Subject } from 'rxjs';
import { Stream, StreamConfig } from './rxjs-audio.interfaces';
export declare class AudioStream implements Stream {
    private audio;
    private currentTrack;
    private audioInput;
    private audioEvents;
    private state;
    private stateChange;
    private config;
    constructor(input: Array<any>, config?: StreamConfig | any);
    /**
     * Play the Audio Stream
     */
    play(): void;
    /**
     * Pause the Audio Stream
     */
    pause(): void;
    /**
     * Stop the Audio Stream
     */
    stop(): void;
    /**
     * Select the Next Audio Item in Playlist.
     * But it doens't play it automatically.
     * Run `play()` to play the stream
     */
    next(): void;
    /**
     * Select the Previous Audio Item in Playlist.
     * But it doens't play it automatically.
     * Run `play()` to play the stream
     */
    previous(): void;
    /**
     * Method to Check if First Track is Playing or not
     * Alternatively you can `this.state()` Observable to get state.
     * state has `isFirstTrack:boolean` to check if First Track is playing or not
     */
    isFirstPlaying(): boolean;
    /**
     * Method to Check if Last Track is Playing or not.
     * Alternatively you can `this.state()` Observable to get state.
     * state has `isLastTrack:boolean` to check if First Track is playing or not
     */
    isLastPlaying(): boolean;
    /**
     * Method to play track at particular index
     */
    playAt(index: number): void;
    /**
     * Switch the currrent Track to particular index
     * It just switch the track, doesn't play the track.
     * To do switch and play, use `platAt()` method.
     */
    switchTo(index: number): void;
    /**
     * Seek audio to particular time
     *
     * @param time
     */
    seekTo(time: number): void;
    /**
     * Returns Observable which listens to audio events of audio stream
     */
    events(): Observable<any>;
    /**
     * Method to get stream of state updates
     */
    getState(): Subject<any>;
    /**
     * initialize the audio stream
     * internal implementation, don't use it.
     */
    private initAudio;
    /**
     * method to add events listener for audio stream
     * internal implementation, don't use it.
     */
    private addEvents;
    /**
     * method to remove events listeners for audio stream
     * internal implementation, don't use it.
     */
    private removeEvents;
    /**
     * method to update state from audio events
     * internal implementation, don't use it.
     */
    private updateStateEvents;
    /**
     * method to update state from properties
     * internal implementation, don't use it.
     */
    private updateStateProps;
    /**
     * extract tracks url from Array of tracks
     * internal implementation, don't use it.
     */
    private extractTracks;
    /**
     * autoplay the next track if set in config
     * internal implementation, don't use it.
     */
    private autoPlayNext;
    /**
     * config the stream for initialization
     * internal implementation, don't use it.
     */
    private configureStream;
}
